! partition_descriptors.f90 --- describe partitions and partitionings
!
! Copyright  (C)  2010  Thomas Jahns <jahns@dkrz.de>
!
! Version: 1.0
! Keywords: partition partitioning descriptor
! Author: Thomas Jahns <jahns@dkrz.de>
! Maintainer: Thomas Jahns <jahns@dkrz.de>
! URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!
! Commentary:
!
!
!
! Code:
!
MODULE partition_descriptors
  IMPLICIT NONE
  PRIVATE
  TYPE, PUBLIC :: partition_extent_1d
    INTEGER :: lowbound, size
  END TYPE partition_extent_1d
  TYPE, PUBLIC :: partition_extent_2d
    TYPE(partition_extent_1d) :: x, y
  END TYPE partition_extent_2d
  PUBLIC :: extent_size, extent_start, extent_end, rebased_extent
  INTERFACE extent_size
    MODULE PROCEDURE extent_size_1d
    MODULE PROCEDURE extent_size_2d
  END INTERFACE
  INTERFACE extent_start
    MODULE PROCEDURE extent_start_1d
    MODULE PROCEDURE extent_start_2d
  END INTERFACE
  INTERFACE extent_end
    MODULE PROCEDURE extent_end_1d
    MODULE PROCEDURE extent_end_2d
  END INTERFACE
  INTERFACE rebased_extent
    MODULE PROCEDURE rebased_extent_1d
  END INTERFACE
CONTAINS
  ELEMENTAL FUNCTION extent_size_1d(extent)
    TYPE(partition_extent_1d), INTENT(in) :: extent
    INTEGER :: extent_size_1d
    extent_size_1d = extent%size
  END FUNCTION extent_size_1d
  ELEMENTAL FUNCTION extent_size_2d(extent)
    TYPE(partition_extent_2d), INTENT(in) :: extent
    INTEGER :: extent_size_2d
    extent_size_2d = extent%x%size * extent%y%size
  END FUNCTION extent_size_2d
  ELEMENTAL FUNCTION extent_start_1d(extent)
    TYPE(partition_extent_1d), INTENT(in) :: extent
    INTEGER :: extent_start_1d
    extent_start_1d = extent%lowbound
  END FUNCTION extent_start_1d
  PURE FUNCTION extent_start_2d(extent)
    TYPE(partition_extent_2d), INTENT(in) :: extent
    INTEGER :: extent_start_2d(2)
    extent_start_2d = (/ extent%x%lowbound, extent%y%lowbound /)
  END FUNCTION extent_start_2d
  ELEMENTAL FUNCTION extent_end_1d(extent)
    TYPE(partition_extent_1d), INTENT(in) :: extent
    INTEGER :: extent_end_1d
    extent_end_1d = extent%lowbound + extent%size - 1
  END FUNCTION extent_end_1d
  PURE FUNCTION extent_end_2d(extent)
    TYPE(partition_extent_2d), INTENT(in) :: extent
    INTEGER :: extent_end_2d(2)
    extent_end_2d = (/ extent%x%lowbound + extent%x%size - 1, &
         extent%y%lowbound + extent%y%size - 1 /)
  END FUNCTION extent_end_2d

  ELEMENTAL FUNCTION rebased_extent_1d(extent, base) RESULT(norm)
    TYPE(partition_extent_1d) :: norm
    TYPE(partition_extent_1d), INTENT(in) :: extent, base
    norm = partition_extent_1d(extent%lowbound - base%lowbound, extent%size)
  END FUNCTION rebased_extent_1d
END MODULE partition_descriptors
!
! Local Variables:
! license-project-url: "https://www.dkrz.de/redmine/projects/show/scales-ppm"
! license-default: "bsd"
! End:
