! scales_ppm_base.f90 --- Fortran 90 interface to core functions
!
! Copyright  (C)  2010  Thomas Jahns <jahns@dkrz.de>
!
! Version: 1.0
! Keywords: core wrapper
! Author: Thomas Jahns <jahns@dkrz.de>
! Maintainer: Thomas Jahns <jahns@dkrz.de>
! URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!
! Commentary:
!
! Look into core.c for functionality common to Fortran and C parts of PPM
! Also provides extra functionality for concise expressions.
!
! Code:
!
MODULE scales_ppm_base
  IMPLICIT NONE
#ifdef USE_MPI
  INCLUDE 'mpif.h'
#else
  INTEGER, PARAMETER :: mpi_comm_world = 0
#endif
  PRIVATE
  PUBLIC :: abort_ppm, assertion
CONTAINS
  SUBROUTINE abort_ppm(msg, source, line, comm)
    CHARACTER(len=*), INTENT(in) :: source, msg
    INTEGER, INTENT(in) :: line
    INTEGER, OPTIONAL, INTENT(in) :: comm
    INTEGER :: comm_dummy
    comm_dummy = MERGE(comm, mpi_comm_world, PRESENT(comm))
    CALL abort_ppm_f(comm_dummy, msg, source, line)
  END SUBROUTINE abort_ppm

  SUBROUTINE assertion(cond, origfile, origline)
    LOGICAL, INTENT(in) :: cond
    CHARACTER(len=*), OPTIONAL, INTENT(in) :: origfile
    INTEGER, OPTIONAL, INTENT(in) :: origline
    CHARACTER(len=255) :: file
    INTEGER :: line

    IF (PRESENT(origfile)) THEN
      file = origfile
    ELSE
      file = 'unknown'
    ENDIF
    IF (PRESENT(origline)) THEN
      line = origline
    ELSE
      line = -1
    ENDIF
    IF (.NOT. cond) CALL abort_ppm('assertion failed', TRIM(file), line)
  END SUBROUTINE assertion

END MODULE scales_ppm_base
!
! Local Variables:
! license-project-url: "https://www.dkrz.de/redmine/projects/show/scales-ppm"
! license-default: "bsd"
! End:
